/* /////////////////////////////////////////////////////////////////////////////
 * File:        comstl_bstr_functions.h (originally MOBStrFn.h, ::SynesisCom)
 *
 * Purpose:     Contains classes and functions for dealing with BSTR strings.
 *
 * Created:
 * Updated:     2nd July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/comstl
 *                          http://www.comstl.org/
 *
 *              email:      submissions@comstl.org  for submissions
 *                          admin@comstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _COMSTL_INCL_H_COMSTL_BSTR_FUNCTIONS
#define _COMSTL_INCL_H_COMSTL_BSTR_FUNCTIONS

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _COMSTL_VER_H_COMSTL_BSTR_FUNCTIONS_MAJOR      1
#define _COMSTL_VER_H_COMSTL_BSTR_FUNCTIONS_MINOR      3
#define _COMSTL_VER_H_COMSTL_BSTR_FUNCTIONS_REVISION   2
#define _COMSTL_VER_H_COMSTL_BSTR_FUNCTIONS_EDIT       25
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* ////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _COMSTL_INCL_H_COMSTL
 #include "comstl.h"                // Include the COMSTL root header
#endif /* !_COMSTL_INCL_H_COMSTL */
#ifndef _STLSOFT_INCL_H_STLSOFT_AUTO_BUFFER
 #include "stlsoft_auto_buffer.h"   // stlsoft::auto_buffer
#endif /* !_STLSOFT_INCL_H_STLSOFT_AUTO_BUFFER */
#ifndef _COMSTL_INCL_H_COMSTL_TASK_ALLOCATOR
 #include "comstl_task_allocator.h" // task_allocator
#endif /* !_COMSTL_INCL_H_COMSTL_TASK_ALLOCATOR */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 *
 * The COMSTL components are contained within the comstl namespace. This is
 * actually an alias for stlsoft::comstl_project,
 *
 * The definition matrix is as follows:
 *
 * _STLSOFT_NO_NAMESPACE    _COMSTL_NO_NAMESPACE    comstl definition
 * ---------------------    --------------------    -----------------
 *  not defined              not defined             = stlsoft::comstl_project
 *  not defined              defined                 not defined
 *  defined                  not defined             comstl
 *  defined                  defined                 not defined
 *
 */

#ifndef _COMSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::comstl */
namespace comstl
{
 #else
/* Define stlsoft::comstl_project */

namespace stlsoft
{

namespace comstl_project
{

 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_COMSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

/// \defgroup comstl_bstr_functions BSTR Functions
/// \ingroup COMSTL
/// \brief These functions create, manipulate and destroy instances of COM's BSTR type
/// @{

/* /////////////////////////////////////////////////////////////////////////////
 * Functions
 */

/// \brief Creates a BSTR from a Unicode string
///
/// \param s The string from which to create the BSTR
/// \result The created BSTR
inline BSTR bstr_create_w(cs_char_w_t const *s)
{
    return ::SysAllocString(s);
}

/// \brief Creates a BSTR from an ANSI string
///
/// \param s The string from which to create the BSTR
/// \result The created BSTR
inline BSTR bstr_create_a(cs_char_a_t const *s)
{
    BSTR    bstr;

    if(s == NULL)
    {
        bstr = NULL;
    }
    else
    {
        stlsoft_ns_qual(auto_buffer)<cs_char_w_t, task_allocator<cs_char_w_t>, 512> buffer(1 + strlen(s));

        ::MultiByteToWideChar(0, 0, s, -1, buffer, buffer.size());

        bstr = bstr_create_w(buffer);
    }

    return bstr;
}

/// \brief Creates a BSTR from an ANSI string
///
/// \param s The string from which to create the BSTR
/// \result The created BSTR
inline BSTR bstr_create(cs_char_a_t const *s)
{
    return bstr_create_a(s);
}

/// \brief Creates a BSTR from a Unicode string
///
/// \param s The string from which to create the BSTR
/// \result The created BSTR
inline BSTR bstr_create(cs_char_w_t const *s)
{
    return bstr_create_w(s);
}

/// \brief Destroys a BSTR
///
/// \param bstr The BSTR to destroy
inline void bstr_destroy(BSTR bstr)
{
    ::SysFreeString(bstr);
}

/// \brief Duplicates a BSTR
///
/// \param bstr The BSTR to duplicate
/// \return The copied BSTR
inline BSTR bstr_dup(BSTR bstr)
{
    return bstr_create(bstr);
}

/* ////////////////////////////////////////////////////////////////////////// */

/// @} // end of group comstl_bstr_functions

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _COMSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
} // namespace comstl
 #else
} // namespace comstl_project
} // namespace stlsoft
 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_COMSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !_COMSTL_INCL_H_COMSTL_BSTR_FUNCTIONS */

/* ////////////////////////////////////////////////////////////////////////// */
