/* /////////////////////////////////////////////////////////////////////////////
 * File:        comstl_rot_functions.h (originally MORotFns.h, ::SynesisCom)
 *
 * Purpose:     COM ROT (Running Object Table) functions.
 *
 * Created:     21st October 1998
 * Updated:     2nd July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/comstl
 *                          http://www.comstl.org/
 *
 *              email:      submissions@comstl.org  for submissions
 *                          admin@comstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _COMSTL_INCL_H_COMSTL_MEMORY_FUNCTIONS
#define _COMSTL_INCL_H_COMSTL_MEMORY_FUNCTIONS

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _COMSTL_VER_H_COMSTL_MEMORY_FUNCTIONS_MAJOR     2
#define _COMSTL_VER_H_COMSTL_MEMORY_FUNCTIONS_MINOR     0
#define _COMSTL_VER_H_COMSTL_MEMORY_FUNCTIONS_REVISION  1
#define _COMSTL_VER_H_COMSTL_MEMORY_FUNCTIONS_EDIT      28
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* ////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _COMSTL_INCL_H_COMSTL
 #include "comstl.h"                // Include the COMSTL root header
#endif /* !_COMSTL_INCL_H_COMSTL */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 *
 * The COMSTL components are contained within the comstl namespace. This is
 * actually an alias for stlsoft::comstl_project,
 *
 * The definition matrix is as follows:
 *
 * _STLSOFT_NO_NAMESPACE    _COMSTL_NO_NAMESPACE    comstl definition
 * ---------------------    --------------------    -----------------
 *  not defined              not defined             = stlsoft::comstl_project
 *  not defined              defined                 not defined
 *  defined                  not defined             comstl
 *  defined                  defined                 not defined
 *
 */

#ifndef _COMSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::comstl */
namespace comstl
{
 #else
/* Define stlsoft::comstl_project */

namespace stlsoft
{

namespace comstl_project
{

 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_COMSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

/// \defgroup comstl_rot_functions Running Object Table (ROT) Functions
/// \ingroup COMSTL
/// \brief These functions create, manipulate and destroy instances of COM's Running Object Table (ROT)
/// @{

/* /////////////////////////////////////////////////////////////////////////////
 * Functions
 */

/// Registers an object in the Running Object Table
///
/// Registers an object and its identifying moniker in the Running Object Table (ROT).
/// 
/// \param grfFlags Registration options
/// \param punkObject Pointer to the object being registered
/// \param pmkObjectName Pointer to the moniker of the object being registered
/// \param pdwRegister Pointer to the value identifying the registration
/// \return An HRESULT indicating success or failure
inline HRESULT Rot_Register(DWORD       grfFlags,
                            LPUNKNOWN   punkObject,
                            LPMONIKER   pmkObjectName,
                            DWORD       *pdwRegister)
{
    LPRUNNINGOBJECTTABLE    prot;
    HRESULT                 hr = ::GetRunningObjectTable(0, &prot);

    if(SUCCEEDED(hr))
    {
        hr = prot->Register(grfFlags, punkObject, pmkObjectName, pdwRegister);

        prot->Release();
    }

    return hr;
}

/// Removes an object from the Running Object Table
///
/// Removes from the Running Object Table (ROT) an entry that was previously
/// registered by a call to Rot_Register().
/// 
/// \param dwRegister Value identifying registration to be revoked
/// \return An HRESULT indicating success or failure
inline HRESULT Rot_Revoke(DWORD dwRegister)
{
    LPRUNNINGOBJECTTABLE    prot;
    HRESULT                 hr = ::GetRunningObjectTable(0, &prot);

    if(SUCCEEDED(hr))
    {
        hr = prot->Revoke(dwRegister);

        prot->Release();
    }

    return hr;
}

/// Determines if object current in the Running Object Table
///
/// Determines whether the object identified by the specified moniker is
/// currently running. This method looks for the moniker in the Running Object
/// Table (ROT).
/// 
/// \param pmkObjectName Pointer to the moniker of the object whose status is desired
/// \return An HRESULT indicating success or failure
inline HRESULT Rot_IsRunning(LPMONIKER pmkObjectName)
{
    LPRUNNINGOBJECTTABLE    prot;
    HRESULT                 hr = ::GetRunningObjectTable(0, &prot);

    if(SUCCEEDED(hr))
    {
        hr = prot->IsRunning(pmkObjectName);

        prot->Release();
    }

    return hr;
}

/// Retrieves the object from the Running Object Table
///
/// Determines whether the object identified by the specified moniker is running,
/// and if it is, retrieves a pointer to that object. This method looks for the
/// moniker in the Running Object Table (ROT), and retrieves the pointer
/// registered there.
/// 
/// \param pmkObjectName Pointer to the moniker of the object
/// \param ppunkObject Address of output variable that receives the IUnknown interface pointer
/// \return An HRESULT indicating success or failure
inline HRESULT Rot_GetObject(   LPMONIKER   pmkObjectName,
                                LPUNKNOWN   *ppunkObject)
{
    LPRUNNINGOBJECTTABLE    prot;
    HRESULT                 hr = ::GetRunningObjectTable(0, &prot);

    if(SUCCEEDED(hr))
    {
        hr = prot->GetObject(pmkObjectName, ppunkObject);

        prot->Release();
    }

    return hr;
}

/// Retrieves the last modification time of a running object in the Running Object Table
///
/// Records the time that a running object was last modified. The object must
/// have previously been registered with the Running Object Table (ROT). This
/// method stores the time of last change in the ROT.
/// 
/// \param dwRegister Value identifying registration being updated
/// \param lpfiletime Pointer to structure containing object's last change time
/// \return An HRESULT indicating success or failure
inline HRESULT Rot_NoteChangeTime(  DWORD       dwRegister,
                                    FILETIME    *lpfiletime)
{
    LPRUNNINGOBJECTTABLE    prot;
    HRESULT                 hr = ::GetRunningObjectTable(0, &prot);

    if(SUCCEEDED(hr))
    {
        hr = prot->NoteChangeTime(dwRegister, lpfiletime);

        prot->Release();
    }

    return hr;
}

/// Retrieves the last modification time of an object in the Running Object Table
///
/// Returns the time that an object was last modified. The object must have
/// previously been registered with the Running Object Table (ROT). This method
/// looks for the last change time recorded in the ROT.
/// 
/// \param pmkObjectName Pointer to moniker on the object whose status is desired
/// \param lpfiletime Pointer to structure containing object's last change time
/// \return An HRESULT indicating success or failure
inline HRESULT Rot_GetTimeOfLastChange( LPMONIKER   pmkObjectName,
                                        FILETIME    *lpfiletime)
{
    LPRUNNINGOBJECTTABLE    prot;
    HRESULT                 hr = ::GetRunningObjectTable(0, &prot);

    if(SUCCEEDED(hr))
    {
        hr = prot->GetTimeOfLastChange(pmkObjectName, lpfiletime);

        prot->Release();
    }

    return hr;
}

/// Queries the current set of objects in the Running Object Table
///
/// Creates and returns a pointer to an enumerator that can list the monikers of
/// all the objects currently registered in the Running Object Table (ROT).
/// 
/// \param ppenumMoniker Address of output variable that receives the IEnumMoniker interface pointer
/// \return An HRESULT indicating success or failure
inline HRESULT Rot_EnumRunning(IEnumMoniker **ppenumMoniker)
{
    LPRUNNINGOBJECTTABLE    prot;
    HRESULT                 hr = ::GetRunningObjectTable(0, &prot);

    if(SUCCEEDED(hr))
    {
        hr = prot->EnumRunning(ppenumMoniker);

        prot->Release();
    }

    return hr;
}

/// @} // end of group comstl_rot_functions

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _COMSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
} // namespace comstl
 #else
} // namespace stlsoft::comstl_project
} // namespace stlsoft
 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_COMSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !_COMSTL_INCL_H_COMSTL_MEMORY_FUNCTIONS */

/* ////////////////////////////////////////////////////////////////////////// */
