/* ////////////////////////////////////////////////////////////////////////////
 * File:        comstl_thread_marshal.h
 *
 * Purpose:     Thread marshalling functions.
 *
 * Created:     25th May 2002
 * Updated:     2nd July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/comstl
 *                          http://www.comstl.org/
 *
 *              email:      submissions@comstl.org  for submissions
 *                          admin@comstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _COMSTL_INCL_H_COMSTL_THREAD_MARSHAL
#define _COMSTL_INCL_H_COMSTL_THREAD_MARSHAL

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _COMSTL_VER_H_COMSTL_THREAD_MARSHAL_MAJOR       1
#define _COMSTL_VER_H_COMSTL_THREAD_MARSHAL_MINOR       2
#define _COMSTL_VER_H_COMSTL_THREAD_MARSHAL_REVISION    2
#define _COMSTL_VER_H_COMSTL_THREAD_MARSHAL_EDIT        22
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* ////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _COMSTL_INCL_H_COMSTL
 #include "comstl.h"                    // Include the COMSTL root header
#endif /* !_COMSTL_INCL_H_COMSTL */
#ifndef _COMSTL_INCL_H_COMSTL_INTERFACE_TRAITS
 #include "comstl_interface_traits.h"   // Include IID_traits
#endif /* !_COMSTL_INCL_H_COMSTL_INTERFACE_TRAITS */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 *
 * The COMSTL components are contained within the comstl namespace. This is
 * actually an alias for stlsoft::comstl_project,
 *
 * The definition matrix is as follows:
 *
 * _STLSOFT_NO_NAMESPACE    _COMSTL_NO_NAMESPACE    comstl definition
 * ---------------------    --------------------    -----------------
 *  not defined              not defined             = stlsoft::comstl_project
 *  not defined              defined                 not defined
 *  defined                  not defined             comstl
 *  defined                  defined                 not defined
 *
 */

#ifndef _COMSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::comstl */
namespace comstl
{
 #else
/* Define stlsoft::comstl_project */

namespace stlsoft
{

namespace comstl_project
{

 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_COMSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

/// \defgroup thread_marshal_functions Thread Marshal Functions
/// \ingroup COMSTL
/// \brief These functions assist in the marshaling of components between threads and processes
/// @{

/* ////////////////////////////////////////////////////////////////////////////
 * Functions
 */

/// \brief Marshal a interface to a stream
///
/// This function marshals the given interface into a new stream instance, which
/// is returned to the caller. The stream may then be passed directly to another
/// thread in the process, from which GetInterfaceAndReleaseStream() may be
/// called.
///
/// \param pitf The interface pointer to marshal
/// \param ppstm A pointer to an IStream pointer to receive the stream
/// \return An HRESULT indicating success or failure
/// \retval E_OUTOFMEMORY Sufficient memory could not be acquired
/// \retval S_OK The operation completed successfully
template <cs_typename_param_k I>
inline HRESULT MarshalInterThreadInterfaceInStream(I *pitf, LPSTREAM *ppstm)
{
    return ::CoMarshalInterThreadInterfaceInStream(IID_traits<I>().iid(), pitf, ppstm);
}

/// \brief Retrieve a marshaled interface pointer from a stream
///
/// This function loads a serialised marshalled interface pointer from the given
/// stream, queries for the interface of the given pointer, and returns the
/// pointer if successful, and an error code if not. The stream is always
/// released, irrespective of the success status of the function as a whole.
///
/// \param pstm An IStream pointer from which object is to be unmarshaled
/// \param ppitf A pointer to the interface pointer to be unmarshaled
/// \return An HRESULT indicating success or failure
/// \retval E_INVALIDARG The argument was invalid
/// \retval S_OK The operation completed successfully

template <cs_typename_param_k I>
inline HRESULT GetInterfaceAndReleaseStream(LPSTREAM pstm, I **ppitf)
{
    return ::CoGetInterfaceAndReleaseStream(pstm, IID_traits<I>().iid(), reinterpret_cast<void**>(ppitf));
}

/* ////////////////////////////////////////////////////////////////////////// */

/// @} // end of group thread_marshal_functions

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _COMSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
} // namespace comstl
 #else
} // namespace comstl_project
} // namespace stlsoft
 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_COMSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !_COMSTL_INCL_H_COMSTL_THREAD_MARSHAL */

/* ////////////////////////////////////////////////////////////////////////// */
