/* /////////////////////////////////////////////////////////////////////////////
 * File:        winstl_file_path_buffer.h
 *
 * Purpose:     Contains the basic_file_path_buffer template class, and ANSI
 *              and Unicode specialisations thereof.
 *
 * Created:     7th February 2002
 * Updated:     2nd July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/winstl
 *                          http://www.winstl.org/
 *
 *              email:      submissions@winstl.org  for submissions
 *                          admin@winstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _WINSTL_INCL_H_WINSTL_FILE_PATH_BUFFER
#define _WINSTL_INCL_H_WINSTL_FILE_PATH_BUFFER

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _WINSTL_VER_H_WINSTL_FILE_PATH_BUFFER_MAJOR     1
#define _WINSTL_VER_H_WINSTL_FILE_PATH_BUFFER_MINOR     4
#define _WINSTL_VER_H_WINSTL_FILE_PATH_BUFFER_REVISION  7
#define _WINSTL_VER_H_WINSTL_FILE_PATH_BUFFER_EDIT      30
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _WINSTL_INCL_H_WINSTL
 #include "winstl.h"                        // Include the WinSTL root header
#endif /* !_WINSTL_INCL_H_WINSTL */
#ifndef _STLSOFT_INCL_H_STLSOFT_AUTO_BUFFER
 #include "stlsoft_auto_buffer.h"           // stlsoft::auto_buffer
#endif /* !_STLSOFT_INCL_H_STLSOFT_AUTO_BUFFER */
#ifndef _WINSTL_INCL_H_WINSTL_PROCESSHEAP_ALLOCATOR
 #include "winstl_processheap_allocator.h"  // winstl::processheap_allocator
#endif /* !_WINSTL_INCL_H_WINSTL_PROCESSHEAP_ALLOCATOR */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::winstl */
namespace winstl
{
 #else
/* Define stlsoft::winstl_project */

namespace stlsoft
{

namespace winstl_project
{

 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

/// \weakgroup libraries STLSoft Libraries
/// \brief The individual libraries

/// \weakgroup libraries_filesystem File-System Library
/// \ingroup libraries
/// \brief This library provides facilities for defining and manipulating file-system objects

/// \weakgroup winstl_filesystem_library File-System Library (WinSTL)
/// \ingroup WinSTL libraries_filesystem
/// \brief This library provides facilities for defining and manipulating file-system objects for the Win32 API
/// @{

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

// class file_path_buffer_base
/// Worker class for basic_file_path_buffer class
///
/// This class is a non-template class primarily so that separate instantiations
/// are not created for each instantiation of the basic_file_path_buffer.
///
/// This class provides a simple function, which is to provide the maximum path
/// length for the host. This information is then cached due to the static nature
/// of the _get_drivesvar() method, although it can be reset by calling the
/// refresh() method on the buffer class.

class file_path_buffer_base
{
public:
    typedef file_path_buffer_base   class_type;

protected:
    /// Returns a mutable 32-bit unsigned integer that acts as the drives variable
    static ws_dword_t &_get_drivesvar() winstl_throw_0()
    {
        static ws_dword_t   s_drivesvar = 0;

        return s_drivesvar;
    }

    /// Calculates the maximum path length of any drive on the system
    static ws_size_t _get_max_pathlen()
    {
        static ws_size_t    s_max_pathlen   =   0;
        ws_dword_t          &_drives        =   _get_drivesvar();
        ws_dword_t          drives          =   ::GetLogicalDrives();

        if( _drives != drives ||
            _drives == 0)
        {
            // Now need to iterate through all
            ws_char_a_t drive[4] = "A:\\";

            for(_drives = drives; !('Z' < drive[0]); ++drive[0], drives >>= 1)
            {
                ws_dword_t  maximum_component_length = 0;

                if( (drives & 0x1) &&
                    ::GetVolumeInformationA(drive, NULL, 0, NULL, &maximum_component_length, NULL, NULL, 0) &&
                    maximum_component_length > s_max_pathlen)
                {
                    s_max_pathlen = maximum_component_length;
                }

                ::GetLastError();
            }
        }

        return s_max_pathlen;
    }
};

// class basic_file_path_buffer
/// Acts as a buffer with sufficient size for any drive on the host machine
///
/// This class is a non-template class primarily so that separate instantiations
/// are not created for each instantiation of the basic_file_path_buffer.
///
/// This class provides a simple function, which is to provide the maximum path
/// length for the host. This information is then cached due to the static nature
/// of the _get_drivesvar() method, although it can be reset by calling the
/// refresh() method on the buffer class.
///
/// \param C The character type
/// \param A The allocator type
/// \param CCH The size of the internal member path structure. On translators that support default template arguments this default to (1 + \c _MAX_PATH)
template<   ws_typename_param_k C
        ,   ws_typename_param_k A
#ifdef __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_FUNDAMENTAL_ARGUMENT_SUPPORT
        ,   ws_size_t CCH = 1 + _MAX_PATH
#else
        ,   ws_size_t CCH /* = 1 + _MAX_PATH */
#endif /* __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_FUNDAMENTAL_ARGUMENT_SUPPORT */
        >
class basic_file_path_buffer
    : protected stlsoft_ns_qual(auto_buffer)<C, A, CCH>
    , protected file_path_buffer_base
{
private:
    typedef stlsoft_ns_qual(auto_buffer)<C, A, CCH>                         auto_buffer_parent_class_type;
    typedef file_path_buffer_base                                           file_path_buffer_base_parent_class_type;
public:
    /// The current parameterisation of the type
    typedef basic_file_path_buffer<C, A, CCH>                               class_type;
    /// The value type
    typedef ws_typename_type_k auto_buffer_parent_class_type::value_type    value_type;
    /// The size type
    typedef ws_typename_type_k auto_buffer_parent_class_type::size_type     size_type;

// Construction
public:
    /// Default constructor
    basic_file_path_buffer()
        : auto_buffer_parent_class_type(1 + _get_max_pathlen())
    {}

// Operations
public:
    /// Returns a mutable (non-const) pointer to the internal buffer
    operator value_type *()
    {
        return auto_buffer_parent_class_type::operator value_type *();
    }
    /// Returns a non-mutable (const) pointer to the internal buffer
    operator value_type const *() const
    {
        return auto_buffer_parent_class_type::data();
    }

    /// Returns the size of the internal buffer
    size_type size() const
    {
        return auto_buffer_parent_class_type::size();
    }

// Operations
public:
    /// Causes the drives to be examined again for the next instance.
    static void refresh() winstl_throw_0()
    {
        file_path_buffer_base_parent_class_type::_get_drivesvar() = 0;
    }

// Not to be implemented
private:
    basic_file_path_buffer(class_type const &rhs);
    const basic_file_path_buffer &operator =(class_type const &rhs);
};

/* Typedefs to commonly encountered types. */
/// Instantiation of the basic_file_path_buffer template for the ANSI character type \c char
typedef basic_file_path_buffer<ws_char_a_t, processheap_allocator<ws_char_a_t>, 1 + _MAX_PATH>  file_path_buffer_a;
/// Instantiation of the basic_file_path_buffer template for the Unicode character type \c wchar_t
typedef basic_file_path_buffer<ws_char_w_t, processheap_allocator<ws_char_w_t>, 1 + _MAX_PATH>  file_path_buffer_w;

/* ////////////////////////////////////////////////////////////////////////// */

/// @} // end of group winstl_filesystem_library

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
} // namespace winstl
 #else
} // namespace winstl_project
} // namespace stlsoft
 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _WINSTL_INCL_H_WINSTL_FILE_PATH_BUFFER */

/* ////////////////////////////////////////////////////////////////////////// */
