/* /////////////////////////////////////////////////////////////////////////////
 * File:        winstl_window_redraw_scope.h (originally MWWndLck.h; ::SynesisWin)
 *
 * Purpose:     Window redraw-state scoping class.
 *
 * Created:     5th January 1996
 * Updated:     2nd July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/winstl
 *                          http://www.winstl.org/
 *
 *              email:      submissions@winstl.org  for submissions
 *                          admin@winstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _WINSTL_INCL_H_WINSTL_WINDOW_REDRAW_SCOPE
#define _WINSTL_INCL_H_WINSTL_WINDOW_REDRAW_SCOPE

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _WINSTL_VER_H_WINSTL_WINDOW_REDRAW_SCOPE_MAJOR      2
#define _WINSTL_VER_H_WINSTL_WINDOW_REDRAW_SCOPE_MINOR      8
#define _WINSTL_VER_H_WINSTL_WINDOW_REDRAW_SCOPE_REVISION   4
#define _WINSTL_VER_H_WINSTL_WINDOW_REDRAW_SCOPE_EDIT       45
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _WINSTL_INCL_H_WINSTL
 #include "winstl.h"                // Include the WinSTL root header
#endif /* _WINSTL_INCL_H_WINSTL */
#ifndef _WINSTL_INCL_H_WINSTL_WINDOW_ACCESS
 #include "winstl_window_access.h"  // winstl::window_access
#endif /* _WINSTL_INCL_H_WINSTL_WINDOW_ACCESS */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::winstl */
namespace winstl
{
 #else
/* Define stlsoft::winstl_project */

namespace stlsoft
{

namespace winstl_project
{

 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

/// \weakgroup libraries STLSoft Libraries
/// \brief The individual libraries

/// \weakgroup libraries_window Window Library
/// \ingroup libraries
/// \brief This library provides facilities for defining and manipulating GUI windows

/// \defgroup winstl_window_library Window Library (WinSTL)
/// \ingroup WinSTL libraries_window
/// \brief This library provides facilities for defining and manipulating Win32 GUI windows
/// @{

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

// window_redraw_scope
/// Provides scoping of the redraw status of a window.
///
/// This class provides scoping of the redraw status of a window via the
/// WM_SETREDRAW message.
class window_redraw_scope
{
public:
    typedef window_redraw_scope class_type;

// Construction
public:
    /// Attempts to lock the given window from redrawing.
    ///
    /// Takes a HWND and changes it's current redraw-status, which is set back to
    /// the original state in the destructor.
    ///
    /// \param wnd The window whose redraw state is to be controlled
    /// \param bInvalidateOnUnlock Determines whether the window is invalidated upon destruction
#ifdef __STLSOFT_CF_MEMBER_TEMPLATE_CTOR_SUPPORT
    template <typename W>
    ws_explicit_k window_redraw_scope(W &wnd, ws_bool_t bInvalidateOnUnlock = true)
        : m_hwnd(get_hwnd(wnd))
#else
    ws_explicit_k window_redraw_scope(HWND wnd, ws_bool_t bInvalidateOnUnlock = true)
        : m_hwnd(wnd)
#endif /* __STLSOFT_CF_MEMBER_TEMPLATE_CTOR_SUPPORT */
        , m_bInvalidateOnUnlock(bInvalidateOnUnlock)
    {
        ::SendMessage(m_hwnd, WM_SETREDRAW, false, 0L);
    }

    /// Resets the redraw status, and invalidates the window, if requested in the constructor
    ~window_redraw_scope() winstl_throw_0()
    {
        ::SendMessage(m_hwnd, WM_SETREDRAW, true, 0L);

        if(m_bInvalidateOnUnlock)
        {
            ::InvalidateRect(m_hwnd, NULL, true);
        }
    }

// Members
private:
    HWND        m_hwnd;
    ws_bool_t   m_bInvalidateOnUnlock;

// Not to be implemented
private:
    window_redraw_scope(window_redraw_scope const &rhs);
    window_redraw_scope const &operator =(window_redraw_scope const &rhs);
};

/* ////////////////////////////////////////////////////////////////////////// */

/// @} // end of group winstl_window_library

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
} // namespace winstl
 #else
} // namespace winstl_project
} // namespace stlsoft
 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _WINSTL_INCL_H_WINSTL_WINDOW_REDRAW_SCOPE */

/* ////////////////////////////////////////////////////////////////////////// */
