/* /////////////////////////////////////////////////////////////////////////////
 * File:        winstl_windows_directory.h
 *
 * Purpose:     Simple class that gets, and makes accessible, the windows
 *              directory.
 *
 * Created:     10th December 2002
 * Updated:     2nd July 2003
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/winstl
 *                          http://www.winstl.org/
 *
 *              email:      submissions@winstl.org  for submissions
 *                          admin@winstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _WINSTL_INCL_H_WINSTL_WINDOWS_DIRECTORY
#define _WINSTL_INCL_H_WINSTL_WINDOWS_DIRECTORY

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
#define _WINSTL_VER_H_WINSTL_WINDOWS_DIRECTORY_MAJOR    1
#define _WINSTL_VER_H_WINSTL_WINDOWS_DIRECTORY_MINOR    4
#define _WINSTL_VER_H_WINSTL_WINDOWS_DIRECTORY_REVISION 1
#define _WINSTL_VER_H_WINSTL_WINDOWS_DIRECTORY_EDIT     18
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _WINSTL_INCL_H_WINSTL
 #include "winstl.h"                    // Include the WinSTL root header
#endif /* !_WINSTL_INCL_H_WINSTL */
#ifndef _WINSTL_INCL_H_WINSTL_FILESYSTEM_TRAITS
 #include "winstl_filesystem_traits.h"  // file_traits
#endif /* !_WINSTL_INCL_H_WINSTL_FILESYSTEM_TRAITS */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::winstl */
namespace winstl
{
 #else
/* Define stlsoft::winstl_project */

namespace stlsoft
{

namespace winstl_project
{

 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

/// \weakgroup libraries STLSoft Libraries
/// \brief The individual libraries

/// \weakgroup libraries_filesystem File-System Library
/// \ingroup libraries
/// \brief This library provides facilities for defining and manipulating file-system objects

/// \weakgroup winstl_filesystem_library File-System Library (WinSTL)
/// \ingroup WinSTL libraries_filesystem
/// \brief This library provides facilities for defining and manipulating file-system objects for the Win32 API
/// @{

/* /////////////////////////////////////////////////////////////////////////////
 * basic_windows_directory
 *
 * This class wraps the GetWindowsDirectory() API function, and effectively acts
 * as a C-string of its value.
 */

/// Represents the windows directory
///
/// \param C The character type
/// \param T The traits type. On translators that support default template arguments, this defaults to filesystem_traits<C>
template<   ws_typename_param_k C
#ifdef __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
        ,   ws_typename_param_k T = filesystem_traits<C>
#else
        ,   ws_typename_param_k T /* = filesystem_traits<C> */
#endif /* __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT */
        >
class basic_windows_directory
{
public:
    /// The char type
    typedef C                               char_type;
    /// The traits type
    typedef T                               traits_type;
    /// The current parameterisation of the type
    typedef basic_windows_directory<C, T>   class_type;
    /// The size type
    typedef ws_size_t                       size_type;

// Operations
public:
    /// Gets the windows directory into the given buffer
    static size_type   get_path(ws_char_a_t *buffer, size_type cchBuffer);
    /// Gets the windows directory into the given buffer
    static size_type   get_path(ws_char_w_t *buffer, size_type cchBuffer);

// Attributes
public:
    /// Returns a non-mutable (const) pointer to the path
    char_type const *get_path() const;
    /// Returns the length of the converted path
    size_type       length() const;

// Conversions
public:
    /// Implicit conversion to a non-mutable (const) pointer to the path
    operator char_type const *() const
    {
        return get_path();
    }

// Implementation
private:
    struct Information
    {
        char_type   m_dir[_MAX_PATH];
        size_type   m_len;

        Information()
            : m_len(get_path(m_dir, winstl_num_elements(m_dir)))
        {}
    };

#if !defined(__STLSOFT_COMPILER_IS_DMC)
    static Information const &_get_information()
    {
        static Information  s_info;

        return s_info;
    }
#else
    // Digital Mars gets an internal compiler error when the
    // preferred implementation is used, so we provide this
    // slightly less efficient implementation.
    Information m_information;
    Information const &_get_information() const
    {
        return m_information;
    }
#endif /* __STLSOFT_COMPILER_IS_DMC */
};

/* /////////////////////////////////////////////////////////////////////////////
 * Typedefs for commonly encountered types
 */

/// Instantiation of the basic_windows_directory template for the ANSI character type \c char
typedef basic_windows_directory<ws_char_a_t, filesystem_traits<ws_char_a_t> >   windows_directory_a;
/// Instantiation of the basic_windows_directory template for the Unicode character type \c wchar_t
typedef basic_windows_directory<ws_char_w_t, filesystem_traits<ws_char_w_t> >   windows_directory_w;
/// Instantiation of the basic_windows_directory template for the Win32 character type \c TCHAR
typedef basic_windows_directory<TCHAR, filesystem_traits<TCHAR> >               windows_directory;

/* /////////////////////////////////////////////////////////////////////////////
 * Implementation
 */

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION

template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        >
inline /* static */ ws_typename_type_k basic_windows_directory<C, T>::size_type basic_windows_directory<C, T>::get_path(ws_char_a_t *buffer, ws_typename_type_k basic_windows_directory<C, T>::size_type cchBuffer)
{
    return static_cast<size_type>(::GetWindowsDirectoryA(buffer, cchBuffer));
}

template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        >
inline /* static */ ws_typename_type_k basic_windows_directory<C, T>::size_type basic_windows_directory<C, T>::get_path(ws_char_w_t *buffer, ws_typename_type_k basic_windows_directory<C, T>::size_type cchBuffer)
{
    return static_cast<size_type>(::GetWindowsDirectoryW(buffer, cchBuffer));
}

template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        >
inline ws_typename_type_k basic_windows_directory<C, T>::char_type const *basic_windows_directory<C, T>::get_path() const
{
#if defined(__STLSOFT_COMPILER_IS_DMC)
    return _get_information().m_dir;
#else
    static char_type const    *s_sz  =   _get_information().m_dir;

    return s_sz;
#endif /* __STLSOFT_COMPILER_IS_DMC */
}

template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        >
inline ws_typename_type_k basic_windows_directory<C, T>::size_type basic_windows_directory<C, T>::length() const
{
    return _get_information().m_len;
}

#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* ////////////////////////////////////////////////////////////////////////// */

/// @} // end of group winstl_filesystem_library

/* /////////////////////////////////////////////////////////////////////////////
 * Shims
 */

/// \weakgroup concepts STLSoft Concepts

/// \weakgroup concepts_shims Shims
/// \ingroup concepts 

/// \weakgroup concepts_shims_string_access String Access Shims
/// \ingroup concepts_shims
/// \brief These \ref concepts_shims "shims" retrieve the C-string for arbitrary types

/// \weakgroup winstl_string_access_shims String Access Shims (WinSTL)
/// \ingroup WinSTL concepts_shims_string_access
/// \brief These \ref concepts_shims "shims" retrieve the C-string for arbitrary types
/// @{

/// \brief Returns the corresponding C-string pointer of the basic_windows_directory \c sd, or a null pointer
template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        >
inline C const *c_str_ptr_null(basic_windows_directory<C, T> const &sd)
{
    return sd;
}

/// \brief Returns the corresponding C-string pointer of the basic_windows_directory \c sd
template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        >
inline C const *c_str_ptr(basic_windows_directory<C, T> const &sd)
{
    return sd;
}

/// \brief Returns the length (in characters) of the basic_windows_directory \c sd, <b><i>not</i></b> including the null-terminating character
template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        >
inline ws_size_t c_str_len(basic_windows_directory<C, T> const &sd)
{
    return sd.length();
}

/// \brief Returns the size (in bytes) of the basic_windows_directory \c sd, <b><i>not</i></b> including the null-terminating character
template<   ws_typename_param_k C
        ,   ws_typename_param_k T
        >
inline ws_size_t c_str_size(basic_windows_directory<C, T> const &sd)
{
    return sd.length() * sizeof(C);
}

/* ////////////////////////////////////////////////////////////////////////// */

/// @} // end of group winstl_string_access_shims

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _WINSTL_NO_NAMESPACE
 #ifdef _STLSOFT_NO_NAMESPACE
} // namespace winstl
 #else
} // namespace winstl_project
} // namespace stlsoft
 #endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _WINSTL_INCL_H_WINSTL_WINDOWS_DIRECTORY */

/* ////////////////////////////////////////////////////////////////////////// */
